@@ -109,6 +109,9 @@ BrowserMainWindow::BrowserMainWindow(QWidget *parent, Qt::WindowFlags flags)
     , m_historyForward(0)
     , m_stop(0)
     , m_reload(0)
+#ifndef QT_NO_PRINTER
+    , m_currentPrinter(nullptr)
+#endif
 {
     setToolButtonStyle(Qt::ToolButtonFollowStyle);
     setAttribute(Qt::WA_DeleteOnClose, true);
@@ -312,6 +315,8 @@ void BrowserMainWindow::setupMenu()
     fileMenu->addSeparator();
 #if defined(QWEBENGINEPAGE_PRINT)
     fileMenu->addAction(tr("P&rint Preview..."), this, SLOT(slotFilePrintPreview()));
+#endif
+#ifndef QT_NO_PRINTER
     fileMenu->addAction(tr("&Print..."), this, SLOT(slotFilePrint()), QKeySequence::Print);
 #endif
     fileMenu->addAction(tr("&Print to PDF..."), this, SLOT(slotFilePrintToPDF()));
@@ -352,7 +357,6 @@ void BrowserMainWindow::setupMenu()
     QAction *m_find = editMenu->addAction(tr("&Find"));
     m_find->setShortcuts(QKeySequence::Find);
     connect(m_find, SIGNAL(triggered()), this, SLOT(slotEditFind()));
-    new QShortcut(QKeySequence(Qt::Key_Slash), this, SLOT(slotEditFind()));
 
     QAction *m_findNext = editMenu->addAction(tr("&Find Next"));
     m_findNext->setShortcuts(QKeySequence::FindNext);
@@ -404,7 +408,10 @@ void BrowserMainWindow::setupMenu()
     viewMenu->addAction(tr("Reset &Zoom"), this, SLOT(slotViewResetZoom()), QKeySequence(Qt::CTRL | Qt::Key_0));
 
     viewMenu->addSeparator();
-    viewMenu->addAction(tr("Page S&ource"), this, SLOT(slotViewPageSource()), tr("Ctrl+Alt+U"));
+    QAction *m_pageSource = viewMenu->addAction(tr("Page S&ource"));
+    m_pageSource->setShortcut(QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_U));
+    m_tabWidget->addWebAction(m_pageSource, QWebEnginePage::ViewSource);
+
     QAction *a = viewMenu->addAction(tr("&Full Screen"), this, SLOT(slotViewFullScreen(bool)),  Qt::Key_F11);
     a->setCheckable(true);
 
@@ -420,13 +427,31 @@ void BrowserMainWindow::setupMenu()
 
     m_historyBack = new QAction(tr("Back"), this);
     m_tabWidget->addWebAction(m_historyBack, QWebEnginePage::Back);
-    m_historyBack->setShortcuts(QKeySequence::Back);
+    QList<QKeySequence> backShortcuts = QKeySequence::keyBindings(QKeySequence::Back);
+    for (auto it = backShortcuts.begin(); it != backShortcuts.end();) {
+        // Chromium already handles navigate on backspace when appropriate.
+        if ((*it)[0] == Qt::Key_Backspace)
+            it = backShortcuts.erase(it);
+        else
+            ++it;
+    }
+    // For some reason Qt doesn't bind the dedicated Back key to Back.
+    backShortcuts.append(QKeySequence(Qt::Key_Back));
+    m_historyBack->setShortcuts(backShortcuts);
     m_historyBack->setIconVisibleInMenu(false);
     historyActions.append(m_historyBack);
 
     m_historyForward = new QAction(tr("Forward"), this);
     m_tabWidget->addWebAction(m_historyForward, QWebEnginePage::Forward);
-    m_historyForward->setShortcuts(QKeySequence::Forward);
+    QList<QKeySequence> fwdShortcuts = QKeySequence::keyBindings(QKeySequence::Forward);
+    for (auto it = fwdShortcuts.begin(); it != fwdShortcuts.end();) {
+        if (((*it)[0] & Qt::Key_unknown) == Qt::Key_Backspace)
+            it = fwdShortcuts.erase(it);
+        else
+            ++it;
+    }
+    fwdShortcuts.append(QKeySequence(Qt::Key_Forward));
+    m_historyForward->setShortcuts(fwdShortcuts);
     m_historyForward->setIconVisibleInMenu(false);
     historyActions.append(m_historyForward);
 
@@ -693,10 +718,10 @@ void BrowserMainWindow::slotFilePrintPreview()
 
 void BrowserMainWindow::slotFilePrint()
 {
-#if defined(QWEBENGINEPAGE_PRINT)
+#ifndef QT_NO_PRINTER
     if (!currentTab())
         return;
-    printRequested(currentTab()->page()->mainFrame());
+    printRequested(currentTab()->page());
 #endif
 }
 
@@ -730,16 +755,29 @@ void BrowserMainWindow::slotFilePrintToPDF()
     currentTab()->page()->printToPdf(invoke(this, &BrowserMainWindow::slotHandlePdfPrinted), dialog->pageLayout());
 }
 
-#if defined(QWEBENGINEPAGE_PRINT)
-void BrowserMainWindow::printRequested(QWebEngineFrame *frame)
+#ifndef QT_NO_PRINTER
+void BrowserMainWindow::slotHandlePagePrinted(bool result)
+{
+    Q_UNUSED(result);
+
+    delete m_currentPrinter;
+    m_currentPrinter = nullptr;
+}
+
+
+void BrowserMainWindow::printRequested(QWebEnginePage *page)
 {
 #ifndef QT_NO_PRINTDIALOG
-    QPrinter printer;
-    QPrintDialog *dialog = new QPrintDialog(&printer, this);
-    dialog->setWindowTitle(tr("Print Document"));
-    if (dialog->exec() != QDialog::Accepted)
+    if (m_currentPrinter)
         return;
-    frame->print(&printer);
+    m_currentPrinter = new QPrinter();
+    QScopedPointer<QPrintDialog> dialog(new QPrintDialog(m_currentPrinter, this));
+    dialog->setWindowTitle(tr("Print Document"));
+    if (dialog->exec() != QDialog::Accepted) {
+        slotHandlePagePrinted(false);
+        return;
+    }
+    page->print(m_currentPrinter, invoke(this, &BrowserMainWindow::slotHandlePagePrinted));
 #endif
 }
 #endif
@@ -850,20 +888,6 @@ void BrowserMainWindow::slotViewFullScreen(bool makeFullScreen)
     }
 }
 
-void BrowserMainWindow::slotViewPageSource()
-{
-    if (!currentTab())
-        return;
-
-    QPlainTextEdit *view = new QPlainTextEdit;
-    view->setWindowTitle(tr("Page Source of %1").arg(currentTab()->title()));
-    view->setMinimumWidth(640);
-    view->setAttribute(Qt::WA_DeleteOnClose);
-    view->show();
-
-    currentTab()->page()->toHtml(invoke(view, &QPlainTextEdit::setPlainText));
-}
-
 void BrowserMainWindow::slotHome()
 {
     QSettings settings;
